{*******************************************************************************

		Programmer: Joel Jean
    Date: 21/04/1998
    Copyright  1998

    Unfolding Dialog Button Component...

    This button automatically provides the "Unfolding dialog box" functionality
    found in some Windows programs. When you drop the button on the form, it
    will resize the form at application startup to just below the bottom of
    the button (You can change that distance with the AdditinalSpace property).
    When the button is pressed, the dialog unfolds to be the size it was in
    the designer.

    This component is freeware... bring any modification to it has needed.
    Please report any changes so I could update this component.

    Please send comments to: jeanj@nb.sympatico.ca
                             joel.jean@hrdc-drhc.gc.ca

*******************************************************************************}
unit UnfloldButton;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

const
	DEFAULT_ADDITIONAL_SPACE = 32;

type
  TButtonCaption = (bcViewDetails, bcHideDetails);
  TUnfoldButtonMode = (bmUnFolded, bmFolded);

  TUnfoldButton = class(TBitBtn)
  private
    { Private declarations }
    FAdditionalSpace: integer;
    InitialFormHeight: integer;
    FoldedFormHeight: integer;
    FButtonMode: TUnfoldButtonMode;
	  FOnFold: TNotifyEvent;
  	FOnUnFold: TNotifyEvent;
    procedure SetAdditionalSpace(Value: integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
    procedure Loaded; override;
  published
    { Published declarations }
    property AdditionalSpace: integer read FAdditionalSpace write SetAdditionalSpace default DEFAULT_ADDITIONAL_SPACE;
    property ButtonMode: TUnfoldButtonMode read FButtonMode write FButtonMode;
    property OnFold: TNotifyEvent read FOnFold write FOnFold;
  	property OnUnFold: TNotifyEvent read FOnUnFold write FOnUnFold;
  end;

procedure Register;

implementation

const
  BUTTON_CAPTION: array[Low(TButtonCaption)..High(TButtonCaption)]
                  of String = ('Details >>', '<< Details');

procedure TUnfoldButton.SetAdditionalSpace(Value: integer);
begin
	if (Value <> FAdditionalSpace) then
    FAdditionalSpace := Value
end;

constructor TUnfoldButton.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);

  { Set Defaults }
  Caption := BUTTON_CAPTION[bcViewDetails];
  ButtonMode := bmFolded;
  FAdditionalSpace := DEFAULT_ADDITIONAL_SPACE;

  { We hate to set a property this isn't our own but if autoscroll is on }
  { the button won't work... }
  (Owner as TForm).AutoScroll := False;
end;

procedure TUnfoldButton.Click;
begin
	inherited Click;

  { Check which state is currently viewed... }
  if (FButtonMode = bmFolded) then
  begin
	  (Owner as TControl).Height := InitialFormHeight;
	  Caption := BUTTON_CAPTION[bcHideDetails];
    FButtonMode := bmUnFolded;

		{ Check if the method is associated to a pointer... }
    if (Assigned(FOnUnFold)) then
    	OnUnFold( Self )
  end
  else
  begin
	  (Owner as TControl).Height := FoldedFormHeight;
	  Caption := BUTTON_CAPTION[bcViewDetails];
    FButtonMode := bmFolded;

		{ Check if the method is associated to a pointer... }
    if (Assigned(FOnFold)) then
    	OnFold( Self )
  end;
end;

procedure TUnfoldButton.Loaded;
var
	ButtonPoint: TPoint;

begin
	inherited Loaded;

  { Save the original height of the form. Get the coords of the button on the }
  { form and afterwards we have to set the new height for the form based on }
  { the button. }
  InitialFormHeight := (Owner as TControl).Height;
  ButtonPoint := Self.ClientOrigin;
  FoldedFormHeight := ButtonPoint.y - (Owner as TControl).ClientOrigin.y +
                       Self.Height + FAdditionalSpace;

	{Determines the status of the form depending on the property settings }
	if (ButtonMode = bmFolded) then
	  (Owner as TControl).Height := FoldedFormHeight
  else
	  Caption := BUTTON_CAPTION[bcHideDetails];
end;



procedure Register;
begin
  RegisterComponents('Custom Components', [TUnfoldButton]);
end;


end.
